import logging
import os
from contextlib import suppress
from typing import Any, Dict, Final, Optional, Type, TypeVar

from typed_envs import registry
from typed_envs._env_var import EnvironmentVariable
from typed_envs.registry import _register_new_env
from typed_envs.typing import StringConverter, VarName


T = TypeVar("T")


class EnvVarFactory:
    """Factory for creating :class:`EnvironmentVariable` instances with optional prefix."""

    def __init__(self, env_var_prefix: Optional[str] = None) -> None:
        """
        Initializes the :class:`EnvVarFactory` with an optional prefix for environment variables.

        Args:
            env_var_prefix: An optional string prefix to be added to environment variable names.
        """
        self.prefix: Final = env_var_prefix
        self.__use_prefix: Final = env_var_prefix is not None
        self.__default_string_converters: Final[Dict[Type, StringConverter]] = {}

    @property
    def default_string_converters(self) -> Dict[Type, StringConverter]:
        return self.__default_string_converters.copy()

    @property
    def use_prefix(self) -> bool:
        return self.__use_prefix

    def create_env(
        self,
        env_var_name: str,
        env_var_type: Type[T],
        default: Any,
        *init_args,
        string_converter: Optional[StringConverter] = None,
        verbose: bool = True,
        **init_kwargs: Any,
    ) -> "EnvironmentVariable[T]":
        """
        Creates a new :class:`EnvironmentVariable` object with the specified parameters.

        Args:
            env_var_name: The name of the environment variable.
            env_var_type: The type of the environment variable.
            default: The default value for the environment variable.
            *init_args: Additional positional arguments for initialization.
            string_converter: An optional callable to convert the string value from the environment.
            verbose: If True, logs the environment variable details.
            **init_kwargs: Additional keyword arguments for initialization.

        Returns:
            An instance of :class:`EnvironmentVariable` with the specified type and value.

        Example:
            Create an environment variable with an integer type using an `EnvVarFactory` instance:

            ```python
            from typed_envs.factory import EnvVarFactory
            factory = EnvVarFactory()
            some_var = factory.create_env("SET_WITH_THIS_ENV", int, 10)

            >>> isinstance(some_var, int)
            True
            >>> isinstance(some_var, EnvironmentVariable)
            True
            ```

            Differences between `some_var` and `int(10)`:
            - `some_var` will type check as both `int` and :class:`EnvironmentVariable`.
            - `some_var.__repr__()` includes contextual information about the :class:`EnvironmentVariable`.

            ```python
            >>> some_var
            <EnvironmentVariable[name=`SET_WITH_THIS_ENV`, type=int, default_value=10, current_value=10, using_default=True]>
            >>> str(some_var)
            "10"
            >>> some_var + 5
            15
            >>> 20 / some_var
            2
            ```

        See Also:
            - :func:`typed_envs.create_env` for creating environment variables without a prefix.
        """
        # Validate name
        if not isinstance(env_var_name, str):
            raise TypeError("env_var_name must be string, not {env_var_name}")

        if not env_var_name:
            raise ValueError("env_var_name must not be empty")

        # Get full name
        if self.__use_prefix:
            full_name = VarName(f"{self.prefix}_{env_var_name}")
        else:
            full_name = VarName(env_var_name)

        # Get value
        var_value = os.environ.get(full_name)
        using_default = var_value is None
        var_value = var_value or default
        if env_var_type is bool:
            if isinstance(var_value, str) and var_value.lower() == "false":
                var_value = False
            else:
                with suppress(ValueError):
                    # if var_value is "0" or "1"
                    var_value = int(var_value)
                var_value = bool(var_value)
        if any(iter_typ in env_var_type.__bases__ for iter_typ in [list, tuple, set]):
            var_value = var_value.split(",")

        # Convert value, if applicable
        if string_converter is None:
            string_converter = self.__default_string_converters.get(env_var_type)
        if string_converter is not None and not (
            using_default and isinstance(default, env_var_type)
        ):
            var_value = string_converter(var_value)

        # Create environment variable
        instance = EnvironmentVariable[env_var_type](  # type: ignore [valid-type]
            var_value, *init_args, **init_kwargs
        )
        # Set additional attributes
        instance._init_arg0 = var_value
        instance._env_name = full_name
        instance._default_value = default
        instance._using_default = using_default

        # Finish up
        if verbose:
            # This code prints envs on script startup for convenience of your users.
            try:
                logger.info(instance.__repr__())
            except RecursionError:
                logger.debug(
                    "unable to properly display your `%s` %s env due to RecursionError",
                    full_name,
                    instance.__class__.__base__,
                )
                with suppress(RecursionError):
                    logger.debug(
                        "Here is your `%s` env in string form: %s",
                        full_name,
                        str(instance),
                    )
        _register_new_env(full_name, instance)
        return instance

    def register_string_converter(
        self, register_for: Type, converter: StringConverter
    ) -> None:
        if register_for in self.__default_string_converters:
            raise ValueError(
                f"There is already a string converter registered for {register_for}"
            )
        elif not callable(converter):
            raise ValueError("converter must be callable")
        self.__default_string_converters[register_for] = converter


# NOTE: While we create the TYPEDENVS_SHUTUP object in the ENVIRONMENT_VARIABLES file as an example,
#       we cannot use it here without creating a circular import.

logger: Final[logging.Logger] = logging.getLogger("typed_envs")

from typed_envs import ENVIRONMENT_VARIABLES

if ENVIRONMENT_VARIABLES.SHUTUP:
    logger.disabled = True
else:
    if not logger.hasHandlers():
        logger.addHandler(logging.StreamHandler())
    if not logger.isEnabledFor(logging.INFO):
        logger.setLevel(logging.INFO)


default_factory: Final[EnvVarFactory] = EnvVarFactory()



from typing import Final

from typed_envs._env_var import EnvironmentVariable
from typed_envs.typing import EnvRegistry, VarName


ENVIRONMENT: Final[EnvRegistry] = EnvRegistry({})
_ENVIRONMENT_VARIABLES_SET_BY_USER: Final[EnvRegistry] = EnvRegistry({})
_ENVIRONMENT_VARIABLES_USING_DEFAULTS: Final[EnvRegistry] = EnvRegistry({})


def _register_new_env(name: VarName, instance: EnvironmentVariable) -> None:
    ENVIRONMENT[name] = instance
    if instance._using_default:
        _ENVIRONMENT_VARIABLES_USING_DEFAULTS[name] = instance
    else:
        _ENVIRONMENT_VARIABLES_SET_BY_USER[name] = instance


__all__ = [
    "ENVIRONMENT",
    "_ENVIRONMENT_VARIABLES_SET_BY_USER",
    "_ENVIRONMENT_VARIABLES_USING_DEFAULTS",
]



from typing import TYPE_CHECKING, Any, Callable, Dict, NewType

if TYPE_CHECKING:
    from typed_envs._env_var import EnvironmentVariable


VarName = NewType("VarName", str)
VarValue = NewType("VarValue", str)

EnvRegistry = NewType("EnvRegistry", Dict[VarName, "EnvironmentVariable"])

StringConverter = Callable[[str], Any]



from functools import lru_cache
from typing import Final, Type, TypeVar

from typed_envs._env_var import EnvironmentVariable


T = TypeVar("T")


__TYPED_CLS_DICT_CONSTANTS: Final = {
    "__repr__": EnvironmentVariable.__repr__,
    "__str__": EnvironmentVariable.__str__,
    "__origin__": EnvironmentVariable,
}


@lru_cache(maxsize=None)
def build_subclass(type_arg: Type[T]) -> Type["EnvironmentVariable[T]"]:
    """
    Returns a mixed subclass of `type_arg` and :class:`EnvironmentVariable` that does 2 things:
     - modifies the __repr__ method so its clear an object's value was set with an env var while when inspecting variables
     - ensures the instance will type check as an :class:`EnvironmentVariable` object without losing information about its actual type

    Aside from these two things, subclass instances will function exactly the same as any other instance of `typ`.
    """
    typed_cls_name = f"EnvironmentVariable[{type_arg.__name__}]"
    typed_cls_bases = (int if type_arg is bool else type_arg, EnvironmentVariable)
    typed_cls_dict = {
        **__TYPED_CLS_DICT_CONSTANTS,
        "__args__": type_arg,
        "__module__": type_arg.__module__,
        "__qualname__": f"EnvironmentVariable[{type_arg.__qualname__}]",
        "__doc__": type_arg.__doc__,
    }
    if hasattr(type_arg, "__annotations__"):
        typed_cls_dict["__annotations__"] = type_arg.__annotations__
    if hasattr(type_arg, "__parameters__"):
        typed_cls_dict["__parameters__"] = type_arg.__parameters__

    try:
        return type(typed_cls_name, typed_cls_bases, typed_cls_dict)
    except TypeError as e:
        raise TypeError(
            *e.args,
            typed_cls_name,
            f"bases: {typed_cls_bases}",
            f"typed: {tuple(map(type, typed_cls_bases))}",
        ) from None



import pytest

from typed_envs import EnvVarFactory


def test_register_string_converter():
    class MyType:
        def __init__(self, value: int): ...

    factory = EnvVarFactory("TEST")
    assert not factory.default_string_converters

    factory.register_string_converter(MyType, int)
    assert MyType in factory.default_string_converters
    assert factory.default_string_converters[MyType] is int

    # Cannot register if one is already registered
    with pytest.raises(ValueError):
        factory.register_string_converter(MyType, int)



"""
This module demonstrates the creation of environment variables using the
:class:`~typed_envs.factory.EnvVarFactory` class with a specified prefix.
"""

from typing import TYPE_CHECKING, Final

from typed_envs.factory import EnvVarFactory

if TYPE_CHECKING:
    from typed_envs._env_var import EnvironmentVariable


_factory: Final[EnvVarFactory] = EnvVarFactory("TYPEDENVS")
"""The :class:`~typed_envs.factory.EnvVarFactory` is initialized with the
prefix "TYPEDENVS", which is automatically added to the environment variable
names created by this factory.
"""

SHUTUP: Final["EnvironmentVariable[bool]"] = _factory.create_env(
    "SHUTUP", bool, False, verbose=False
)
"""An environment variable named "TYPEDENVS_SHUTUP" of type :class:`bool`.
It defaults to `False` if not set in the environment. If the environment
variable is set to any non-empty string, it will be interpreted as `True`.

Examples:
    To access the `SHUTUP` environment variable, you can use it as a regular
    boolean:

    >>> from typed_envs.ENVIRONMENT_VARIABLES import SHUTUP
    >>> isinstance(SHUTUP, bool)
    True
    >>> SHUTUP
    <EnvironmentVariable[name=`TYPEDENVS_SHUTUP`, type=bool, default_value=False, current_value=False, using_default=True]>

    If you set the environment variable `TYPEDENVS_SHUTUP` to a non-empty
    string, it will be interpreted as `True`:

    >>> import os
    >>> os.environ['TYPEDENVS_SHUTUP'] = '1'
    >>> from typed_envs.ENVIRONMENT_VARIABLES import SHUTUP
    >>> SHUTUP
    <EnvironmentVariable[name=`TYPEDENVS_SHUTUP`, type=bool, default_value=False, current_value=True, using_default=False]>

See Also:
    - :class:`~typed_envs.factory.EnvVarFactory` for creating environment
      variables with custom prefixes.
    - :class:`~typed_envs._env_var.EnvironmentVariable` for details on how
      environment variables are represented and used.
"""



import asyncio

import pytest

from typed_envs import EnvironmentVariable, create_env


def test_int_env():
    """Test integer environment variable creation and operations.

    Verifies that the environment variable created using :func:`create_env` behaves
    like an instance of :class:`int` and is an instance of :class:`EnvironmentVariable`.
    It checks support for arithmetic operations such as addition, subtraction,
    multiplication, and division.

    See Also:
        - :func:`create_env`
        - :class:`EnvironmentVariable`
    """
    env = create_env("TEST", int, 10)
    assert type(env).__name__ == "EnvironmentVariable[int]"
    assert isinstance(env, int)
    assert isinstance(env, EnvironmentVariable)
    assert isinstance(env, EnvironmentVariable[int])
    env + 10
    env - 10
    env * 10
    env / 10


def test_str_env():
    """Test string environment variable creation and operations.

    Checks that the environment variable behaves like an instance of :class:`str`
    and is an instance of :class:`EnvironmentVariable`. It verifies support for
    string operations like `upper` and `lower`. Also ensures that adding an integer
    to the string environment variable raises a :class:`TypeError`.

    Note:
        If the default value is not a string, it is cast as a string.

    See Also:
        - :func:`create_env`
        - :class:`EnvironmentVariable`
    """
    env = create_env("TEST", str, 10)
    assert type(env).__name__ == "EnvironmentVariable[str]"
    assert isinstance(env, str)
    assert isinstance(env, EnvironmentVariable)
    assert isinstance(env, EnvironmentVariable[str])
    env.upper()
    env.lower()
    env.encode()
    with pytest.raises(TypeError):
        env + 10


def test_complex_env():
    """Test complex environment variable creation with custom type.

    Verifies that the environment variable created is an instance of
    :class:`asyncio.Semaphore` and :class:`EnvironmentVariable`. It checks for the
    presence of `acquire` and `release` methods, specific to semaphore objects.

    See Also:
        - :func:`create_env`
        - :class:`EnvironmentVariable`
        - :class:`asyncio.Semaphore`
    """
    env = create_env("TEST", asyncio.Semaphore, default=10, string_converter=int)
    assert type(env).__name__ == "EnvironmentVariable[Semaphore]"
    assert isinstance(env, asyncio.Semaphore)
    assert isinstance(env, EnvironmentVariable)
    assert isinstance(env, EnvironmentVariable[asyncio.Semaphore])
    assert hasattr(env, "acquire")
    assert hasattr(env, "release")


def test_bool_conversion():
    """Test boolean environment variable conversion behavior.

    Demonstrates that a boolean environment variable behaves like an :class:`int`
    due to subclassing, and cannot be directly checked as an instance of :class:`bool`,
    verified by expecting an assertion failure when checking its instance type.

    Note:
        You can't subclass a bool, so it's the only type that breaks our type checking.
        We subclass :class:`int` as a workaround to create a bool-like class.

    See Also:
        - :func:`create_env`
        - :class:`EnvironmentVariable`
    """
    env = create_env("TEST", bool, default="test")
    assert type(env).__name__ == "EnvironmentVariable[bool]"
    # You can't subclass a bool so its the only type that breaks our type checking
    with pytest.raises(AssertionError):
        assert isinstance(env, bool)
    assert isinstance(env, int)
    assert isinstance(env, EnvironmentVariable[bool])
    assert env


@pytest.mark.parametrize("value", ("0", "false", "False", "FALSE"))
def test_falsey_bool_conversion(value):
    """Test boolean environment variable conversion behavior for falsey values.

    Demonstrates that falsey values are properly converted, the environment variable behaves
    like an :class:`int` due to subclassing, and cannot be directly checked as an instance of
    :class:`bool`, verified by expecting an assertion failure when checking its instance type.

    Note:
        You can't subclass a bool, so it's the only type that breaks our type checking.
        We subclass :class:`int` as a workaround to create a bool-like class.

    See Also:
        - :func:`create_env`
        - :class:`EnvironmentVariable`
    """
    env = create_env("TEST", bool, default=value)
    assert type(env).__name__ == "EnvironmentVariable[bool]"
    # You can't subclass a bool so its the only type that breaks our type checking
    with pytest.raises(AssertionError):
        assert isinstance(env, bool)
    assert isinstance(env, int)
    assert isinstance(env, EnvironmentVariable[bool])
    assert not env



import os
import sys
from setuptools import find_packages, setup
from mypyc.build import mypycify

from typed_envs import description, description_addon

if sys.implementation.name == "cpython":

    MYPYC_DEBUG_LEVEL = os.environ.get("MYPYC_DEBUG_LEVEL", "0")

    paths_to_compile = [
        "typed_envs/__init__.py",
        # TODO: implement a proxy wrapper instead of hacky subclasses "typed_envs/_env_var.py",
        "typed_envs/_typed.py",
        "typed_envs/ENVIRONMENT_VARIABLES.py",
        # TODO: fix mypyc IR error "typed_envs/factory.py",
        "typed_envs/registry.py",
        "typed_envs/typing.py",
        "--pretty",
        "--install-types",
        "--disable-error-code=assignment",
        "--disable-error-code=attr-defined",
    ]

    ext_modules = mypycify(paths=paths_to_compile, debug_level=MYPYC_DEBUG_LEVEL)
else:
    ext_modules = []

setup(
    name="typed_envs",
    url="https://github.com/BobTheBuidler/typed-envs",
    author="BobTheBuidler",
    author_email="bobthebuidlerdefi@gmail.com",
    license="MIT",
    classifiers=[
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.13",
        "Programming Language :: Python :: 3.14",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries",
    ],
    description=description,
    long_description=description + description_addon,
    python_requires=">=3.8,<4",
    packages=find_packages(),
    use_scm_version={
        "root": ".",
        "relative_to": __file__,
        "local_scheme": "no-local-version",
        "version_scheme": "python-simplified-semver",
    },
    setup_requires=["setuptools_scm"],
    package_data={"typed_envs": ["py.typed"]},
    include_package_data=True,
    ext_modules=ext_modules,
)



from functools import lru_cache
from typing import ClassVar, Final, Generic, TypeVar, Type, Any, final


T = TypeVar("T")


# NOTE: though this lib does create specialized subclasses, users should consider this class as @final
@final
class EnvironmentVariable(Generic[T]):
    """
    Base class for creating custom wrapper subclasses on the fly.

    Note:
        This is just a base class used to create custom wrapper subclasses on the fly.
        You must never initialize these directly.
        You must use the :func:`create_env` function either on the main module or on an :class:`EnvVarFactory` to initialize your env vars.

    Example:
        Useful for enhancing type hints and __repr__ of variables that hold values you set with env vars.
        Note: This is just a helper class to create custom wrapper classes on the fly.
        You should never initialize these directly.

        Functionally, :class:`EnvironmentVariable` objects will work exactly the same as any instance of specified `typ`.

        In the example below, `some_var` can be used just like any other `int` object.

        .. code-block:: python

            import typed_envs
            some_var = typed_envs.create_env("SET_WITH_THIS_ENV", int, 10)

            >>> isinstance(some_var, int)
            True
            >>> isinstance(some_var, EnvironmentVariable)
            True

        There are only 2 differences between `some_var` and `int(10)`:
        - `some_var` will properly type check as an instance of both `int` and :class:`EnvironmentVariable`
        - `some_var.__repr__()` will include contextual information about the :class:`EnvironmentVariable`.

        .. code-block:: python

            >>> some_var
            EnvironmentVariable[int](name=`SET_WITH_THIS_ENV`, default_value=10, using_default=True)
            >>> str(some_var)
            "10"
            >>> some_var + 5
            15
            >>> 20 / some_var
            2

    See Also:
        :func:`create_env`, :class:`EnvVarFactory`
    """

    # TODO: give these docstrings
    _default_value: Any
    _using_default: bool
    _env_name: str
    _init_arg0: Any

    __origin__: ClassVar[Type["EnvironmentVariable"]]

    def __init__(self, *args, **kwargs) -> None:
        if type(self) is EnvironmentVariable:
            raise RuntimeError(
                "You should not initialize these directly, please use the factory"
            )
        try:
            super().__init__(*args, **kwargs)
        except TypeError as e:
            expected = "object.__init__() takes exactly one argument (the instance to initialize)"
            if str(e) != expected:
                raise

    def __str__(self) -> str:
        base_type = self.__args__
        string_from_base = base_type.__str__(self)
        # NOTE: If this returns True, base type's `__str__` method calls `__repr__` and our custom `__repr__` breaks it.
        if string_from_base == repr(self):
            # We broke it but it's all good, we can fix it with some special case logic.
            return str(bool(self)) if base_type is bool else base_type.__repr__(self)
        return (
            base_type.__str__(self)
            if "object at 0x" in string_from_base
            else string_from_base
        )

    def __repr__(self) -> str:
        if self._using_default:
            return "EnvironmentVariable[{}](name=`{}`, default_value={}, using_default=True)".format(
                self.__args__.__qualname__,
                self._env_name,
                self._default_value,
            )
        else:
            return "EnvironmentVariable[{}](name=`{}`, default_value={}, current_value={})".format(
                self.__args__.__qualname__,
                self._env_name,
                self._default_value,
                self._init_arg0,
            )

    def __class_getitem__(cls, type_arg: Type[T]) -> Type["EnvironmentVariable[T]"]:
        """
        Returns a mixed subclass of `type_arg` and :class:`EnvironmentVariable` that does 2 things:
         - modifies the __repr__ method so its clear an object's value was set with an env var while when inspecting variables
         - ensures the instance will type check as an :class:`EnvironmentVariable` object without losing information about its actual type

        Aside from these two things, subclass instances will function exactly the same as any other instance of `typ`.
        """
        if cls is EnvironmentVariable:
            return _build_subclass(type_arg)  # type: ignore [arg-type]
        return super().__class_getitem__(type_arg)  # type: ignore [misc]


@lru_cache(maxsize=None)
def _build_subclass(type_arg: Type[T]) -> Type["EnvironmentVariable[T]"]:
    """
    Returns a mixed subclass of `type_arg` and :class:`EnvironmentVariable` that does 2 things:
     - modifies the __repr__ method so its clear an object's value was set with an env var while when inspecting variables
     - ensures the instance will type check as an :class:`EnvironmentVariable` object without losing information about its actual type

    Aside from these two things, subclass instances will function exactly the same as any other instance of `typ`.
    """
    from typed_envs import _typed

    return _typed.build_subclass(type_arg)  # type: ignore [arg-type]



from typing import Any, Optional, Type, TypeVar

from typed_envs._env_var import EnvironmentVariable
from typed_envs.factory import EnvVarFactory, logger, default_factory
from typed_envs.registry import (
    ENVIRONMENT,
    _ENVIRONMENT_VARIABLES_SET_BY_USER,
    _ENVIRONMENT_VARIABLES_USING_DEFAULTS,
)
from typed_envs.typing import StringConverter


description = """\
typed_envs is used to create specialized `EnvironmentVariable` objects that behave exactly the same as any other instance of the `typ` used to create them.

typed_envs is used for:
    - defining your envs in a readable, user friendly way
    - enhancing type hints for the returned instances
    - enhancing __repr__ of the returned instance with extra contextual information
"""

description_addon = """\

In the example below, `some_var` can be used just like as any other `int` object.

```
import typed_envs
some_var = typed_envs.create_env("SET_WITH_THIS_ENV", int, 10)
>>> isinstance(some_var, int)
True
>>> isinstance(some_var, EnvironmentVariable)
True
```

There are only 2 differences between `some_var` and `int(10)`:
    - `some_var` will properly type check as an instance of both `int` and `EnvironmentVariable`
    - `some_var.__repr__()` will include contextual information about the `EnvironmentVariable`.

```
>>> some_var
<EnvironmentVariable[name=`SET_WITH_THIS_ENV`, type=int, default_value=10, current_value=10, using_default=True]>
>>> str(some_var)
"10"
>>> some_var + 5
15
>>> 20 / some_var
2
```

"""

T = TypeVar("T")


def create_env(
    name: str,
    typ: Type[T],
    default: Any,
    *init_args,
    string_converter: Optional[StringConverter] = None,
    verbose: bool = True,
    **init_kwargs
) -> "EnvironmentVariable[T]":
    """
    Returns a new :class:`EnvironmentVariable` object with no prefix specified.

    Functionally, :class:`EnvironmentVariable` objects will work exactly the same as any instance of specified `typ`.

    Args:
        name: The name of the environment variable.
        typ: The type of the environment variable. This determines the type of the value that the environment variable will hold.
        default: The default value for the environment variable if not specified in the current environment.
        *init_args: Additional positional arguments for initialization of the environment variable.
        string_converter: An optional callable to convert the string value from the environment.
        verbose: If True, logs the environment variable details for debugging and informational purposes.
        **init_kwargs: Additional keyword arguments for initialization of the environment variable.

    Examples:
        In the example below, `some_var` can be used just like any other `int` object.

        >>> import typed_envs
        >>> some_var = typed_envs.create_env("SET_WITH_THIS_ENV", int, 10)
        >>> isinstance(some_var, int)
        True
        >>> isinstance(some_var, EnvironmentVariable)
        True

        There are only 2 differences between `some_var` and `int(10)`:
        - `some_var` will properly type check as an instance of both `int` and :class:`EnvironmentVariable`
        - `some_var.__repr__()` will include contextual information about the :class:`EnvironmentVariable`.

        >>> some_var
        <EnvironmentVariable[name=`SET_WITH_THIS_ENV`, type=int, default_value=10, current_value=10, using_default=True]>
        >>> str(some_var)
        "10"
        >>> some_var + 5
        15
        >>> 20 / some_var
        2

    See Also:
        :class:`EnvVarFactory` for creating environment variables with a prefix.
    """
    return default_factory.create_env(
        name,
        typ,
        default,
        *init_args,
        string_converter=string_converter,
        verbose=verbose,
        **init_kwargs
    )


__all__ = [
    "create_env",
    "EnvVarFactory",
    "ENVIRONMENT",
    "_ENVIRONMENT_VARIABLES_SET_BY_USER",
    "_ENVIRONMENT_VARIABLES_USING_DEFAULTS",
]

# Just putting these here so I don't accidentally remove the 'unused' import
EnvironmentVariable
logger
